//--------------------------------------------------------------------
// Flappy
//--------------------------------------------------------------------
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../runtime/dsl2");
require("../watchers/system");
var node_uuid_1 = require("node-uuid");
//--------------------------------------------------------------------
// Utils
//--------------------------------------------------------------------
function toEAVs(eavs, obj) {
    var record = node_uuid_1.v4();
    for (var attribute in obj) {
        var values = obj[attribute];
        if (values.constructor === Array) {
            for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
                var value = values_1[_i];
                eavs.push([record, attribute, value]);
            }
        }
        else {
            eavs.push([record, attribute, values]);
        }
    }
    return eavs;
}
//--------------------------------------------------------------------
// Program
//--------------------------------------------------------------------
var prog = new dsl2_1.Program("flappy");
prog.attach("system");
prog.attach("svg");
prog.attach("html");
//--------------------------------------------------------------------
// Draw the game world
//--------------------------------------------------------------------
prog.commit("draw the game world", function (_a) {
    var find = _a.find, record = _a.record;
    var world = find("world");
    return [
        world.add("tag", "html/div")
            .add("style", record("html/style", { "user-select": "none" }))
            .add("children", [
            record("svg", "game-window", { viewBox: "10 0 80 100", width: 480 }).add("children", [
                record("svg/rect", { x: 0, y: 0, width: 100, height: 53, fill: "rgb(112,197,206)", sort: 0 }),
                record("svg/rect", { x: 0, y: 95, width: 100, height: 5, fill: "rgb(222,216,149)", sort: 0 }),
                record("svg/image", { x: 0, y: 52, width: 100, height: 43, sort: 1, preserveAspectRatio: "xMinYMin slice", href: "https://cdn.rawgit.com/bhauman/flappy-bird-demo/master/resources/public/imgs/background.png" }),
            ])
        ])
    ];
});
//--------------------------------------------------------------------
// menu screens
//--------------------------------------------------------------------
prog.block("draw the main menu", function (_a) {
    var find = _a.find, record = _a.record;
    find("world", { screen: "menu" });
    var svg = find("game-window");
    return [
        svg.add("children", record("svg/text", { x: 50, y: 45, text: "Click the screen to begin!", sort: 10, "font-size": 6, "text-anchor": "middle" }))
    ];
});
prog.block("draw the game over menu", function (_a) {
    var find = _a.find, record = _a.record;
    var _b = find("world", { screen: "game over" }), score = _b.score, best = _b.best;
    var svg = find("game-window");
    return [
        svg.add("children", [
            record("svg/text", { x: 50, y: 30, "text-anchor": "middle", "font-size": 6, text: "Game Over :(", sort: 10 }),
            record("svg/text", { x: 50, y: 55, "text-anchor": "middle", "font-size": 6, text: "Score " + score, sort: 11 }),
            record("svg/text", { x: 50, y: 65, "text-anchor": "middle", "font-size": 6, text: "Best " + best, sort: 12 }),
            record("svg/text", { x: 50, y: 85, "text-anchor": "middle", "font-size": 4, text: "Click to play again!", sort: 13 })
        ])
    ];
});
//--------------------------------------------------------------------
// Score
//--------------------------------------------------------------------
prog.block("calculate the score", function (_a) {
    var find = _a.find, record = _a.record, lib = _a.lib;
    var math = lib.math;
    var world = find("world");
    return [
        world.add("score", math.floor(world.distance))
    ];
});
//--------------------------------------------------------------------
// Start the game
//--------------------------------------------------------------------
prog.commit("clicking starts the game", function (_a) {
    var find = _a.find, record = _a.record, lib = _a.lib, choose = _a.choose;
    var math = lib.math;
    var world = find("world");
    var svg = find("game-window");
    // find("html/event/click", {element:svg});
    find("html/event/click", "html/direct-target");
    choose(function () { world.screen == "menu"; }, function () { world.screen == "game over"; });
    var bestScore = math.max(world.score, world.best);
    var player = find("player");
    return [
        world.remove("screen").add("screen", "game")
            .remove("distance").add("distance", 0)
            .remove("best").add("best", bestScore),
        player.remove("x").add("x", 25)
            .remove("y").add("y", 50)
            .remove("velocity").add("velocity", 0)
    ];
});
//--------------------------------------------------------------------
// Draw the player
//--------------------------------------------------------------------
prog.block("draw the player", function (_a) {
    var find = _a.find, record = _a.record;
    var svg = find("game-window");
    var player = find("player");
    return [
        svg.add("children", [
            record("svg/image", { player: player, width: 10, height: 10, sort: 8, href: "http://i.imgur.com/sp68LtM.gif" })
                .add("x", player.x - 5)
                .add("y", player.y - 5)
        ])
    ];
});
//--------------------------------------------------------------------
// Obstacles
//--------------------------------------------------------------------
prog.block("draw obstacles", function (_a) {
    var find = _a.find, record = _a.record;
    var svg = find("game-window");
    var obstacle = find("obstacle");
    var height = obstacle.height, gap = obstacle.gap;
    var bottomHeight = height + gap;
    var imgs = "https://cdn.rawgit.com/bhauman/flappy-bird-demo/master/resources/public/imgs";
    return [
        svg.add("children", [
            record("svg", "obs-spr", { obstacle: obstacle, sort: 2, overflow: "visible" }).add("children", [
                record("svg/image", { x: 0, y: 0, width: 10, height: height, preserveAspectRatio: "none", href: imgs + "/pillar-bkg.png", sort: 1 }),
                record("svg/image", { x: -1, y: height - 5, width: 12, height: 5, preserveAspectRatio: "none", href: imgs + "/lower-pillar-head.png", sort: 2 }),
                record("svg/image", { x: 0, y: bottomHeight, width: 10, height: 90 - bottomHeight, preserveAspectRatio: "none", href: imgs + "/pillar-bkg.png", sort: 1 }),
                record("svg/image", { x: -1, y: bottomHeight, width: 12, height: 5, preserveAspectRatio: "none", href: imgs + "/lower-pillar-head.png", sort: 2 }),
            ])
                .add("x", obstacle.x)
        ])
    ];
});
prog.block("every 2 distance, a wild obstacle appears", function (_a) {
    var find = _a.find, math = _a.lib.math;
    var distance = find("world").distance;
    var obstacle = find("obstacle");
    var obstacleDistance = distance + obstacle.offset;
    obstacleDistance >= 0;
    var prex = 50 * math.mod(obstacleDistance, 2);
    var x = 100 - prex;
    return [
        obstacle.add("x", x)
    ];
});
// @TODO: What's the deal with airline food?
prog.commit("adjust the height of the gap", function (_a) {
    var find = _a.find, _b = _a.lib, math = _b.math, random = _b.random;
    var frame = find("frames").frame;
    var world = find("world", { screen: "game" });
    frame != world.frame;
    var obstacle = find("obstacle");
    obstacle.x > 90;
    var height = random.number(frame) * 30 + 5;
    return [obstacle.remove('height').add("height", height)];
});
//--------------------------------------------------------------------
// Flapping the player
//--------------------------------------------------------------------
prog.commit("apply a velocity when you click", function (_a) {
    var find = _a.find;
    var world = find("world", { screen: "game" });
    find("html/event/click", "html/direct-target");
    var player = find("player", "self");
    return [
        player.remove("velocity").add("velocity", 1.17)
    ];
});
//--------------------------------------------------------------------
// Scroll the world
//--------------------------------------------------------------------
prog.commit("scroll the world", function (_a) {
    var find = _a.find, not = _a.not;
    var frame = find("frames").frame;
    var world = find("world", { screen: "game" });
    frame != world.frame;
    var player = find("player");
    var adjust = 1 / 30;
    not(function () { find("html/event/click"); });
    return [
        world.remove("frame").add("frame", frame)
            .remove("distance").add("distance", world.distance + adjust),
        player.remove("y").add("y", player.y - player.velocity)
            .remove("velocity").add("velocity", player.velocity + world.gravity)
    ];
});
//--------------------------------------------------------------------
// Collision
//--------------------------------------------------------------------
prog.commit("collide with ground", function (_a) {
    var find = _a.find;
    var world = find("world", { screen: "game" });
    var player = find("player");
    player.y > 85; // ground height + player radius
    return [
        world.remove("screen", "game").add("screen", "game over")
    ];
});
// Hangs due to union/choose
prog.commit("collide with obstacle", function (_a) {
    var find = _a.find, choose = _a.choose, math = _a.lib.math;
    var world = find("world", { screen: "game" });
    var player = find("player");
    var obstacle = find("obstacle");
    var dx = math.abs(obstacle.x + 5 - player.x) - 10;
    dx < 0;
    choose(function () { player.y - 5 <= obstacle.height; }, function () { player.y + 5 >= obstacle.gap + obstacle.height; });
    return [
        world.remove("screen", "game").add("screen", "game over")
    ];
});
//--------------------------------------------------------------------
// svg/html translation
//--------------------------------------------------------------------
prog
    .block("Translate elements into html", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("html/div");
    return [elem.add("tag", "html/element").add("tagname", "div")];
});
prog
    .block("Translate elements into svg", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("svg");
    return [elem.add("tag", "svg/element").add("tagname", "svg")];
})
    .block("Translate elements into svg", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("svg/rect");
    return [elem.add("tag", "svg/element").add("tagname", "rect")];
})
    .block("Translate elements into svg", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("svg/text");
    return [elem.add("tag", "svg/element").add("tagname", "text")];
})
    .block("Translate elements into svg", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("svg/image");
    return [elem.add("tag", "svg/element").add("tagname", "image")];
});
//--------------------------------------------------------------------
// Go!
//--------------------------------------------------------------------
var changes = [];
toEAVs(changes, { tag: ["frames", "system/timer"], resolution: 33.333333333333 });
toEAVs(changes, { tag: ["player", "self"], name: "eve", x: 25, y: 50, velocity: 0 });
toEAVs(changes, { tag: "world", screen: "menu", frame: 0, distance: 0, best: 0, gravity: -0.061 });
toEAVs(changes, { tag: "obstacle", gap: 35, offset: 0 });
toEAVs(changes, { tag: "obstacle", gap: 35, offset: -1 });
// console.profile("flappy");
prog.inputEavs(changes);
// prog.inputEavs([["meep", "tag", "frames"], ["meep", "frame", 1]])
// prog.inputEavs(toEAVs([], {tag: ["html/event/click", "html/direct-target"]}));
// prog.inputEavs([["meep", "frame", 2], ["meep", "frame", 1, -1]])
// prog.inputEavs([["meep", "frame", 3], ["meep", "frame", 2, -1]])
// prog.inputEavs(toEAVs([], {tag: ["html/event/click", "html/direct-target"]}));
// prog.inputEavs([["meep", "frame", 4], ["meep", "frame", 3, -1]])
// prog.inputEavs([["meep", "frame", 5], ["meep", "frame", 4, -1]])
// prog.inputEavs(toEAVs([], {tag: ["html/event/click", "html/direct-target"]}));
// prog.inputEavs([["meep", "frame", 6], ["meep", "frame", 5, -1]])
// prog.inputEavs([["meep", "frame", 7], ["meep", "frame", 6, -1]])
// prog.inputEavs([["meep", "frame", 8], ["meep", "frame", 7, -1]])
// for(let ix = 9; ix < 60; ix++) {
//   prog.inputEavs([["meep", "frame", ix], ["meep", "frame", ix-1, -1]])
//   prog.inputEavs(toEAVs([], {tag: ["html/event/click", "html/direct-target"]}));
// }
// for(let ix = 60; ix < 100; ix++) {
//   prog.inputEavs([["meep", "frame", ix], ["meep", "frame", ix-1, -1]])
// }
//   prog.inputEavs(toEAVs([], {tag: ["html/event/click", "html/direct-target"]}));
// // for(let ix = 100; ix < 200; ix++) {
// //   prog.inputEavs([["meep", "frame", ix], ["meep", "frame", ix-1, -1]])
// // }
// console.profileEnd();
// let reporter = new PerformanceReporter();
// reporter.report(prog.context.tracker);
console.log(prog);
//# sourceMappingURL=flappy.js.map