//--------------------------------------------------------------------
// Editor
//--------------------------------------------------------------------
"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("../watchers/watcher");
var node_uuid_1 = require("node-uuid");
//--------------------------------------------------------------------
// Fixture constants
//--------------------------------------------------------------------
var EDITOR_ID = "|" + node_uuid_1.v4();
var STYLE_ID = "|" + node_uuid_1.v4();
var TAG_MARINA_ID = "|" + node_uuid_1.v4();
var TAG_MARINARA_ID = "|" + node_uuid_1.v4();
var BLOCK_PPL_W_BOATS_ID = "|" + node_uuid_1.v4();
var BLOCK_BOAT_TYPES_ID = "|" + node_uuid_1.v4();
var FRAME_PPL_W_BOATS_QUERY_ID = "|" + node_uuid_1.v4();
//--------------------------------------------------------------------
// Watcher
//--------------------------------------------------------------------
var EditorWatcher = (function (_super) {
    __extends(EditorWatcher, _super);
    function EditorWatcher() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    EditorWatcher.prototype.setup = function () {
        this.editor = this.createEditor();
        var _a = this, editor = _a.editor, program = _a.program;
        this.program
            .watch("Export all tags.", function (_a) {
            var find = _a.find, string = _a.lib.string, not = _a.not, record = _a.record;
            var rec = find();
            var tag = rec.tag;
            not(function () { return string.index_of(tag, "editor/") == 0; });
            return [
                record("client_tag", { "client_tag": tag })
            ];
        })
            .asDiffs(watcher_1.forwardDiffs(editor))
            .watch("Send the editor attributes on records matching the tag.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var attributes_from_tag = find("editor/attributes-from-tag");
            var query_tag = attributes_from_tag.query_tag;
            var rec = find({ tag: query_tag });
            var attribute = lookup(rec).attribute;
            return [
                attributes_from_tag.add({ tag: "editor/attributes-from-tag", query_tag: query_tag, attribute: attribute })
            ];
        })
            .asDiffs(watcher_1.forwardDiffs(editor))
            .watch("Send the editor attributes on records matching the parent's children.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var attributes_from_parent = find("editor/attributes-from-parent");
            var parent_tag = attributes_from_parent.parent_tag, parent_attribute = attributes_from_parent.parent_attribute;
            var parent = find({ tag: parent_tag });
            var _b = lookup(parent), par_attr = _b.attribute, rec = _b.value;
            parent_attribute == par_attr;
            var attribute = lookup(rec).attribute;
            return [
                attributes_from_parent.add({ tag: "editor/attributes-from-parent", parent_tag: parent_tag, parent_attribute: parent_attribute, attribute: attribute })
            ];
        })
            .asDiffs(watcher_1.forwardDiffs(editor))
            .watch("Attributes with a | are ids.", function (_a) {
            var find = _a.find, lookup = _a.lookup, string = _a.lib.string, record = _a.record;
            var attributes_from_tag = find("editor/attributes-from-tag");
            var query_tag = attributes_from_tag.query_tag;
            var rec = find({ tag: query_tag });
            var _b = lookup(rec), attribute = _b.attribute, value = _b.value;
            string.index_of(value, "|");
            return [
                attributes_from_tag.add("record_attribute", attribute)
            ];
        })
            .asDiffs(watcher_1.forwardDiffs(editor));
        this.initEditor();
        this.fixtureEditor();
    };
    EditorWatcher.prototype.createEditor = function () {
        var editor = new watcher_1.Program("Editor");
        editor.attach("ui");
        editor.attach("shape");
        var compiler = editor.attach("compiler");
        compiler.injectInto(this.program);
        compiler.registerWatcherFunction("send-to-editor", watcher_1.forwardDiffs(editor, "send-to-editor"));
        //--------------------------------------------------------------------
        // Root UI
        //--------------------------------------------------------------------
        editor
            .block("Draw the root editor view.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            return [
                record("editor/view", "ui/row", { editor: editor }).add("children", [
                    record("editor/nav", "ui/column", { editor: editor, sort: 0 }),
                    record("editor/main", "ui/column", { editor: editor, sort: 1 }).add("children", [
                        record("ui/row", { editor: editor, sort: 0, class: "editor-block-header" }).add("children", [
                            record("editor/block/description", "ui/column", { editor: editor }),
                            record("editor/block/storyboard", "ui/row", { editor: editor })
                        ]),
                        record("ui/row", "editor/block/content", { editor: editor, sort: 1 })
                    ])
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Navigation
        //--------------------------------------------------------------------
        editor
            .block("Populate the nav bar with the program's block tags.", function (_a) {
            var find = _a.find, record = _a.record;
            var nav = find("editor/nav");
            var tag = nav.editor.block.nav_tag;
            return [
                nav.add("children", [
                    record("editor/nav/tag", "ui/column", { editor: nav.editor, sort: tag.name, nav_tag: tag }).add("children", [
                        record("ui/text", { sort: 0, text: tag.name })
                    ])
                ])
            ];
        })
            .block("Populate nav tags with the blocks that have them.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var tag = find("editor/nav/tag");
            var block = tag.editor.block;
            block.nav_tag == tag.nav_tag;
            var name = choose(function () { return block.name; }, function () { return "Untitled Block"; })[0];
            return [
                tag.add("children", [
                    record("editor/nav/block", "ui/text", { editor: tag.editor, nav_tag: tag.nav_tag, block: block, text: name, sort: name })
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Block Description
        //--------------------------------------------------------------------
        editor
            .block("Populate the block description for the active block.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var description = find("editor/block/description");
            var active_block = description.editor.active_block;
            var name = choose(function () { return active_block.name; }, function () { return "Untitled Block"; })[0];
            var text = choose(function () { return active_block.description; }, function () { return ""; })[0];
            return [
                description.add("children", [
                    record("ui/text", { sort: 0, text: name, class: "editor-block-title" }),
                    record("ui/text", { sort: 1, text: text })
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Block Frames
        //--------------------------------------------------------------------
        editor
            .block("Populate the block storyboard for the active block.", function (_a) {
            var find = _a.find, record = _a.record;
            var storyboard = find("editor/block/storyboard");
            var editor = storyboard.editor;
            var active_block = editor.active_block;
            var frame = active_block.storyboard;
            return [
                storyboard.add("children", [
                    record("editor/block/frame", "ui/column", { editor: editor, sort: frame.sort, frame: frame }).add("children", [
                        record("ui/text", { text: frame.type })
                    ])
                ])
            ];
        })
            .block("Mark the active frame.", function (_a) {
            var find = _a.find;
            var editor = find("editor/root");
            var frame = editor.active_frame;
            var frame_elem = find("editor/block/frame", { frame: frame });
            return [
                frame_elem.add("class", "active")
            ];
        })
            .commit("Clicking a frame activates it", function (_a) {
            var find = _a.find;
            var frame_elem = find("editor/block/frame");
            find("html/event/click", { element: frame_elem });
            var frame = frame_elem.frame, editor = frame_elem.editor;
            return [
                editor.remove("active_frame").add("active_frame", frame)
            ];
        })
            .block("Add a new frame button to the storyboard.", function (_a) {
            var find = _a.find, record = _a.record;
            var storyboard = find("editor/block/storyboard");
            var editor = storyboard.editor;
            var active_block = editor.active_block;
            return [
                storyboard.add("children", [
                    record("editor/new-frame", "editor/block/frame", "ui/column", { editor: editor, sort: Infinity })
                ])
            ];
        })
            .commit("Clicking the new frame button opens it", function (_a) {
            var find = _a.find;
            var new_frame = find("editor/new-frame");
            find("html/event/click", "html/direct-target", { element: new_frame });
            return [
                new_frame.add("open", "true")
            ];
        })
            .block("When the new frame is open, display a list of editor types to choose from.", function (_a) {
            var find = _a.find, record = _a.record;
            var new_frame = find("editor/new-frame", { open: "true" });
            var editor = new_frame.editor;
            return [
                new_frame.add("children", [
                    record("editor/new-frame/type", "ui/button", { editor: editor, text: "Query", type: "query", class: "flat" }),
                    record("editor/new-frame/type", "ui/button", { editor: editor, text: "Data", type: "data", class: "flat" }),
                ])
            ];
        })
            .commit("Clicking a new frame type adds a frame of that type and closes the new frame button.", function (_a) {
            var find = _a.find, gather = _a.gather, choose = _a.choose, record = _a.record;
            var new_frame_type = find("editor/new-frame/type");
            find("html/event/click", "html/direct-target", { element: new_frame_type });
            var type = new_frame_type.type, editor = new_frame_type.editor;
            var new_frame = find("editor/new-frame", { editor: editor });
            var block = editor.active_block;
            var ix = choose(function () { return gather(block.storyboard).per(block).count() + 1; }, function () { return 1; })[0];
            return [
                new_frame.remove("open"),
                block.add("storyboard", [
                    record("editor/frame", { block: block, type: type, sort: ix })
                ])
            ];
        });
        this.attachMoleculeGenerator(editor);
        this.attachQueryEditor(editor);
        this.attachDataEditor(editor);
        return editor;
    };
    EditorWatcher.prototype.attachMoleculeGenerator = function (editor) {
        //--------------------------------------------------------------------
        // Molecule generation
        //--------------------------------------------------------------------
        editor.block("Create a set of molecules for the active block's queries.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            var active_block = editor.active_block;
            var frame = active_block.storyboard;
            frame.type == "query";
            var node = frame.node;
            node.tag == "editor/root-node";
            return [
                active_block.add("molecule-watch", [
                    record("editor/molecule/watch", "eve/compiler/block", { editor: editor, frame: frame, name: "Create molecules", type: "watch", watcher: "send-to-editor" }).add("constraint", [
                        record("editor/atom/record", "eve/compiler/record", { node: node, record: record("editor/atom/entity", "eve/compiler/var", { node: node }) }),
                    ])
                ])
            ];
        });
        editor.block("Embed subnodes.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_watch = find("editor/molecule/watch");
            var editor = molecule_watch.editor, frame = molecule_watch.frame;
            var node = frame.node;
            var parent_node = node.parent_node, parent_attribute = node.parent_attribute;
            var parent_record = find("editor/atom/record", { node: parent_node });
            var record_var;
            return [
                record_var = record("editor/atom/entity", "eve/compiler/var", { node: node }),
                parent_record.add("attribute", record({ tag: "eve/compiler/av", attribute: parent_attribute, value: record_var })),
                molecule_watch.add("constraint", [
                    record("editor/atom/record", "eve/compiler/record", { node: node, record: record_var })
                ])
            ];
        });
        editor.block("Attach node query tags to their atom records.", function (_a) {
            var find = _a.find, record = _a.record;
            var atom_record = find("editor/atom/record");
            var node = atom_record.node;
            var query_tag = node.query_tag;
            return [
                atom_record.add("attribute", record({ tag: "eve/compiler/av", attribute: "tag", value: query_tag })),
            ];
        });
        editor.block("Attach node fields to their atom records.", function (_a) {
            var find = _a.find, record = _a.record;
            var atom_record = find("editor/atom/record");
            var node = atom_record.node;
            var query_field = node.query_field;
            return [
                atom_record.add("attribute", [
                    record({
                        tag: "eve/compiler/av",
                        attribute: query_field,
                        value: record("editor/atom/field", "eve/compiler/var", { node: node, query_field: query_field })
                    })
                ])
            ];
        });
        editor.block("Output a molecule for each root node.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_watch = find("editor/molecule/watch");
            var editor = molecule_watch.editor, frame = molecule_watch.frame;
            var node = find("editor/root-node");
            frame.node == node;
            var entity_var = find("editor/atom/entity", { node: node });
            var molecule_var;
            return [
                molecule_var = record("editor/molecule/output_var", "eve/compiler/var", { frame: frame, node: node }),
                molecule_watch.add("constraint", [
                    record("editor/molecule/output", "eve/compiler/output", { molecule_watch: molecule_watch, node: node, record: molecule_var }).add("attribute", [
                        record("eve/compiler/av", { attribute: "tag", value: "editor/molecule" }),
                        record("eve/compiler/av", { attribute: "editor", value: editor }),
                        record("eve/compiler/av", { attribute: "frame", value: frame }),
                        record("eve/compiler/av", { attribute: "node", value: node }),
                        record("eve/compiler/av", { attribute: "root_atom_record", value: entity_var }),
                    ])
                ]),
            ];
        });
        editor.block("Attach the root atom to molecules.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_output = find("editor/molecule/output");
            var molecule_watch = molecule_output.molecule_watch;
            var node = molecule_output.node;
            var atom_var;
            return [
                atom_var = record("editor/atom/output_var", "eve/compiler/var", { node: node }),
                molecule_watch.add("constraint", [
                    record("editor/atom/output", "eve/compiler/output", { node: node, molecule_output: molecule_output, record: atom_var })
                ]),
                molecule_output.add("parent_node", node),
            ];
        });
        editor.block("Attach subnode atoms to molecules.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule_output = find("editor/molecule/output");
            var molecule_watch = molecule_output.molecule_watch, parent_node = molecule_output.parent_node;
            var node = find("editor/query-node");
            node.parent_node == parent_node;
            var atom_var;
            return [
                atom_var = record("editor/atom/output_var", "eve/compiler/var", { node: node }),
                molecule_watch.add("constraint", [
                    record("editor/atom/output", "eve/compiler/output", { node: node, molecule_output: molecule_output, record: atom_var })
                ]),
                molecule_output.add("parent_node", node),
            ];
        });
        editor.block("Fill vital atom output attributes and attach them to their molecule output.", function (_a) {
            var find = _a.find, record = _a.record;
            var atom_output = find("editor/atom/output");
            var molecule_output = atom_output.molecule_output, node = atom_output.node;
            var entity_var = find("editor/atom/entity", { node: node });
            return [
                atom_output.add("attribute", [
                    record("eve/compiler/av", { attribute: "tag", value: "editor/atom" }),
                    record("eve/compiler/av", { attribute: "node", value: node }),
                    record("eve/compiler/av", { attribute: "molecule", value: molecule_output.record }),
                    record("eve/compiler/av", { attribute: "record", value: entity_var }),
                ]),
                molecule_output.add("attribute", [
                    record("eve/compiler/av", "eve/compiler/attribute/non-identity", { attribute: "atom", value: atom_output.record }),
                ]),
            ];
        });
        // @FIXME: What about root node identity?
        editor.block("Attach node query fields to their atom outputs.", function (_a) {
            var find = _a.find, record = _a.record;
            var atom_field_var = find("editor/atom/field");
            var node = atom_field_var.node, query_field = atom_field_var.query_field;
            var atom_output = find("editor/atom/output", { node: node });
            var molecule_output = atom_output.molecule_output;
            var molecule_watch = molecule_output.molecule_watch;
            var field_var;
            return [
                field_var = record("editor/atom/field/output_var", "eve/compiler/var", { node: node, root_node: molecule_output.node }),
                molecule_watch.add("constraint", [
                    record("editor/atom/field/output", "eve/compiler/output", { node: node, record: field_var }).add("attribute", [
                        record("eve/compiler/av", { attribute: query_field, value: atom_field_var })
                    ])
                ]),
                atom_output.add("attribute", [
                    record("eve/compiler/av", "eve/compiler/attribute/non-identity", { attribute: "field", value: field_var }),
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Molecule placement
        //--------------------------------------------------------------------
        editor
            .commit("Molecules start with a seed of 1", function (_a) {
            var find = _a.find, not = _a.not;
            var molecule = find("editor/molecule");
            not(function () { return molecule.seed; });
            return [molecule.add("seed", 1)];
        })
            .block("Find a potential location for new molecules", function (_a) {
            var find = _a.find, _b = _a.lib, random = _b.random, math = _b.math, record = _a.record;
            var molecule = find("editor/molecule");
            var seed = molecule.seed, atom = molecule.atom;
            var x = math.round(random.number(molecule + " " + seed + " x") * 10);
            var y = math.round(random.number(molecule + " " + seed + " y") * 5);
            return [
                molecule.add({ x: x, y: y }) // , positioned: "true"
            ];
        })
            .commit("A molecule with positioned false and a low enough seed should try to reposition.", function (_a) {
            var find = _a.find, record = _a.record;
            var molecule = find("editor/molecule", { positioned: "false" });
            molecule.seed < 4;
            return [
                molecule.remove("positioned").remove("seed").add("seed", molecule.seed + 1),
            ];
        })
            .block("Determine the suitability of a potential molecule position by colliding it's footprint with existing cells.", function (_a) {
            var find = _a.find, choose = _a.choose, not = _a.not;
            var molecule = find("editor/molecule");
            molecule.seed < 4;
            var delay = find("someone-is-maybe-positioned");
            //not(() => molecule.positioned);
            var positioned = choose(function () {
                var skirt = molecule.skirt;
                var other = find("editor/molecule");
                other.seed < 4;
                molecule.frame == other.frame;
                molecule.generation >= other.generation;
                molecule != other;
                var other_atom = other.atom;
                other_atom.x == skirt.x;
                other_atom.y == skirt.y;
                return "false";
            }, function () {
                return "true";
            })[0];
            return [
                molecule.add("positioned", positioned)
            ];
        })
            .block("create a skirt around unpositioned molecules.", function (_a) {
            var find = _a.find, not = _a.not, math = _a.lib.math, record = _a.record;
            var molecule = find("editor/molecule");
            //not(() => molecule.positioned == "true");
            var atom = molecule.atom;
            var ix = find("range").ix;
            var _b = find("spiral", { row: math.mod(atom.y, 2), ix: ix }), x = _b.x, y = _b.y;
            return [molecule.add("skirt", [
                    record("editor/molecule/skirt", { x: atom.x + x, y: atom.y + y })
                ])];
        })
            .block("Sort atoms by id.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var molecule = find("editor/molecule");
            var atom = molecule.atom;
            var ix = gather(atom).per(molecule).sort();
            return [
                atom.add("sort", ix)
            ];
        })
            .commit("When we first see a molecule, mark its generation.", function (_a) {
            var find = _a.find, not = _a.not, choose = _a.choose, gather = _a.gather;
            var molecule = find("editor/molecule");
            not(function () { return molecule.generation; });
            var generation = choose(function () {
                var existing = find("editor/molecule");
                existing.generation;
                return gather(existing).count();
            }, function () { return 1; })[0];
            return [molecule.add("generation", generation)];
        })
            .block("DEBUG: Sort molecules by id.", function (_a) {
            var find = _a.find, gather = _a.gather, record = _a.record;
            var molecule = find("editor/molecule");
            var ix = gather(molecule.frame, molecule.generation, molecule).sort();
            return [
                molecule.add("sort", ix)
            ];
        })
            .block("Compute atom positions from their sort.", function (_a) {
            var find = _a.find, math = _a.lib.math, record = _a.record;
            var molecule = find("editor/molecule");
            var atom = molecule.atom, mol_x = molecule.x, mol_y = molecule.y;
            var _b = find("spiral", { row: math.mod(molecule.y, 2), ix: atom.sort }), x = _b.x, y = _b.y;
            return [
                atom.add({ x: mol_x + x, y: mol_y + y }),
                record("someone-is-maybe-positioned")
            ];
        });
        //--------------------------------------------------------------------
        // Molecule Interaction
        //--------------------------------------------------------------------
        editor
            .commit("Clicking on an atom cell opens it's molecule.", function (_a) {
            var find = _a.find, not = _a.not;
            var atom_cell = find("editor/atom/cell");
            find("html/event/click", { element: atom_cell });
            var molecule = atom_cell.molecule;
            not(function () { return molecule.open == "true"; });
            return [
                molecule.add("open", "true")
            ];
        })
            .commit("Clicking on an atom cell closes any currently open molecules.", function (_a) {
            var find = _a.find, not = _a.not;
            var atom_cell = find("editor/atom/cell");
            find("html/event/click", { element: atom_cell });
            var molecule = atom_cell.molecule;
            var editor = molecule.editor;
            var other_molecule = find("editor/molecule", { editor: editor, open: "true" });
            return [
                other_molecule.remove("open")
            ];
        });
    };
    EditorWatcher.prototype.attachQueryEditor = function (editor) {
        editor.block("When the active frame is a query, inject the query editor UI.", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var content = find("editor/block/content");
            var editor = content.editor;
            editor.active_frame.type == "query";
            return [
                content.add("children", [
                    record("editor/block/query-tree", "ui/column", { editor: editor }),
                    record("editor/block/query-canvas", { editor: editor })
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Block Query Tree
        //--------------------------------------------------------------------
        editor
            .block("Compute the label and color of query nodes.", function (_a) {
            var find = _a.find, choose = _a.choose, string = _a.lib.string;
            var node = find("editor/query-node");
            var name = node.name, sort = node.sort;
            var label = string.uppercase(string.get(name, 1));
            var color = choose(function () { return find("node-color", { ix: sort }).color; }, function () { return "gray"; })[0];
            return [node.add({ label: label, color: color })];
        })
            .block("Populate the block query for the active block.", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var query_elem = find("editor/block/query-tree");
            var editor = query_elem.editor;
            var active_frame = editor.active_frame;
            active_frame.type == "query";
            var node = active_frame.node;
            var main_pattern = union(function () { return node.query_tag; }, function () { return node.parent_attribute; })[0];
            return [
                record("editor/query/node", "ui/row", { editor: editor, sort: node.sort, node: node, frame: active_frame }).add("children", [
                    record("editor/query/hex", "shape/hexagon", { side: 21, lineWidth: 2, strokeStyle: "#AAA", fillStyle: "white", sort: 0, frame: active_frame, node: node }).add("content", [
                        record("ui/text", { text: node.label, style: record({ color: node.color }) })
                    ]),
                    record("editor/query/pattern", "ui/column", { sort: 1, frame: active_frame, node: node }).add("children", [
                        record("editor/query/pattern/main", "ui/row", { sort: 0, node: node }).add("children", [
                            record("ui/text", { sort: 1, text: main_pattern, class: "editor-query-tag" }),
                            record("ui/spacer", { sort: 2 })
                        ])
                    ])
                ])
            ];
        })
            .block("Query root nodes are children of the query.", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var query_elem = find("editor/block/query-tree");
            var root_node = find("editor/query/node", { editor: query_elem.editor, node: find("editor/root-node") });
            return [
                query_elem.add("children", root_node)
            ];
        })
            .block("Non-root nodes are children of their parent node.", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var query_elem = find("editor/block/query-tree");
            var subnode = find("editor/query/node", { editor: query_elem.editor });
            var node = subnode.node;
            var parent_node = node.parent_node;
            var parent_pattern = find("editor/query/pattern", { node: parent_node });
            return [
                parent_pattern.add("children", [
                    record("ui/column", { node: node, sort: 4, class: "editor-query-subnode" }).add("children", subnode)
                ])
            ];
        })
            .block("Query nodes with attributes display them as a tree in the pattern.", function (_a) {
            var find = _a.find, record = _a.record;
            var query_pattern = find("editor/query/pattern");
            var node = query_pattern.node;
            var query_field = node.query_field;
            return [
                query_pattern.add("children", [
                    record("ui/column", { node: node, sort: 2 }).add("children", [
                        record("editor/query/field", "ui/row", { node: node, query_field: query_field, sort: query_field }).add("children", [
                            record("ui/text", { sort: 1, text: query_field }),
                            record("ui/spacer", { sort: 2 })
                        ])
                    ])
                ])
            ];
        })
            .block("When a query node is open, display delete node button.", function (_a) {
            var find = _a.find, record = _a.record;
            var query_node = find("editor/query/node", { open: "true" });
            var node = query_node.node;
            var main = find("editor/query/pattern/main", { node: node });
            return [main.add("children", record("editor/query/delete-node", "ui/button", { sort: 9, node: node, icon: "close-round" }))];
        })
            .commit("Clicking a delete node button removes it from the block", function (_a) {
            var find = _a.find, record = _a.record;
            var delete_node = find("editor/query/delete-node");
            var click = find("html/event/click", { element: delete_node });
            var node = delete_node.node;
            var frame = find("editor/frame", { node: node });
            return [
                node.remove(),
                frame.remove("node", node)
            ];
        })
            .block("When a query node is open, display delete field buttons.", function (_a) {
            var find = _a.find, record = _a.record;
            var query_node = find("editor/query/node", { open: "true" });
            var node = query_node.node;
            var field = find("editor/query/field", { node: node });
            var query_field = field.query_field;
            return [field.add("children", record("editor/query/delete-field", "ui/button", { sort: 9, node: node, query_field: query_field, icon: "close-round" }))];
        })
            .commit("Clicking a delete field button removes its attribute from the pattern", function (_a) {
            var find = _a.find, record = _a.record;
            var delete_field = find("editor/query/delete-field");
            var click = find("html/event/click", { element: delete_field });
            var node = delete_field.node, query_field = delete_field.query_field;
            return [
                node.remove("query_field", query_field)
            ];
        })
            .commit("Clicking a query hex opens the add attribute menu", function (_a) {
            var find = _a.find, not = _a.not;
            var query_hex = find("editor/query/hex");
            var click = find("html/event/click", { element: query_hex });
            var node = query_hex.node;
            var query_node = find("editor/query/node", { node: node });
            not(function () { return query_node.open; });
            return [
                query_node.add("open", "true")
            ];
        })
            .commit("Clicking an open query hex closes it", function (_a) {
            var find = _a.find;
            var query_hex = find("editor/query/hex");
            var click = find("html/event/click", { element: query_hex });
            var node = query_hex.node;
            var query_node = find("editor/query/node", { node: node, open: "true" });
            return [
                query_node.remove("open")
            ];
        })
            .watch("If a query node is adding an attribute, request attributes matching its tag from the client.", function (_a) {
            var find = _a.find, record = _a.record;
            var query_node = find("editor/query/node", { open: "true" });
            var node = query_node.node;
            return [record("editor/attributes-from-tag", { query_tag: node.query_tag })];
        })
            .asDiffs(watcher_1.forwardDiffs(this.program))
            .watch("If a query node is adding an attribute, request attributes matching its position in the hierarchy.", function (_a) {
            var find = _a.find, record = _a.record;
            var query_node = find("editor/query/node", { open: "true" });
            var node = query_node.node;
            return [record("editor/attributes-from-parent", { parent_tag: node.parent_node.query_tag, parent_attribute: node.parent_attribute })];
        })
            .asDiffs(watcher_1.forwardDiffs(this.program))
            .block("When a query node is in the new attribute state, show all the attributes matching its tag", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var query_node = find("editor/query/node", { open: "true" });
            var node = query_node.node;
            var query_pattern = find("editor/query/pattern");
            query_node.children == query_pattern;
            var attribute = choose(function () {
                var attribute = find("editor/attributes-from-tag", { query_tag: node.query_tag }).attribute;
                return [attribute];
            }, function () {
                var attribute = find("editor/attributes-from-parent", { parent_tag: node.parent_node.query_tag, parent_attribute: node.parent_attribute }).attribute;
                return [attribute];
            })[0];
            return [
                query_pattern.add("children", [
                    record("ui/column", { node: node, sort: 3 }).add("children", [
                        record("editor/query/node/new-attribute", "ui/text", { text: attribute, sort: attribute, attribute: attribute, node: node })
                    ])
                ])
            ];
        })
            .commit("Clicking a new attribute in a query node adds it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var new_attribute = find("editor/query/node/new-attribute");
            var click = find("html/event/click", { element: new_attribute });
            var node = new_attribute.node, attribute = new_attribute.attribute;
            var query_node = find("editor/query/node", { node: node });
            not(function () { return find("editor/attributes-from-tag", { query_tag: node.query_tag, record_attribute: attribute }); });
            return [
                node.add("query_field", attribute),
                query_node.remove("open")
            ];
        })
            .commit("Clicking a new record attribute in a query node adds it as a sub-node.", function (_a) {
            var find = _a.find, gather = _a.gather, choose = _a.choose, record = _a.record;
            var new_attribute = find("editor/query/node/new-attribute");
            var click = find("html/event/click", { element: new_attribute });
            var node = new_attribute.node, attribute = new_attribute.attribute;
            var query_node = find("editor/query/node", { node: node });
            find("editor/attributes-from-tag", { query_tag: node.query_tag, record_attribute: attribute });
            var frame = query_node.frame;
            var ix = choose(function () {
                return [gather(frame.node).count() + 1];
            }, function () { return 1; })[0];
            return [
                node.add("query_subnode", attribute),
                query_node.frame.add("node", [
                    record("editor/query-node", "editor/subnode", {
                        type: "join",
                        sort: ix,
                        name: attribute,
                        parent_attribute: attribute,
                        parent_node: node,
                    })
                ]),
                query_node.remove("open")
            ];
        })
            .block("The query always has a new node button", function (_a) {
            var find = _a.find, record = _a.record;
            var query_elem = find("editor/block/query-tree");
            var editor = query_elem.editor;
            var active_frame = editor.active_frame;
            return [
                query_elem.add("children", [
                    record("ui/row", "editor/query/new-node", { sort: 9999, frame: active_frame }).add("children", [
                        record("shape/hexagon", { side: 21, lineWidth: 2, strokeStyle: "#AAA", fillStyle: "white", class: "editor-query-hex" }).add("content", [
                            record("ui/text", { text: "+", style: record({ color: "#AAA", "font-weight": 500 }) })
                        ]),
                    ])
                ])
            ];
        })
            .commit("Clicking on the new node button opens it.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var new_node_button = find("editor/query/new-node");
            not(function () { return new_node_button.open == "true"; });
            var click = find("html/event/click", { element: new_node_button });
            return [
                new_node_button.add("open", "true")
            ];
        })
            .block("When the new node button is open, display a list of the clients tags.", function (_a) {
            var find = _a.find, record = _a.record;
            var new_node_button = find("editor/query/new-node", { open: "true" });
            var tag = find("client_tag").client_tag;
            return [
                new_node_button.add("children", [
                    record("ui/column", { sort: 1 }).add("children", [
                        record("ui/text", "editor/query/new-node/tag", { text: tag, sort: tag, client_tag: tag, new_node_button: new_node_button })
                    ])
                ])
            ];
        })
            .commit("Clicking on a new node tag adds it as a node to the query.", function (_a) {
            var find = _a.find, gather = _a.gather, choose = _a.choose, record = _a.record;
            var new_node_tag = find("editor/query/new-node/tag");
            var new_node_button = new_node_tag.new_node_button;
            var client_tag = new_node_tag.client_tag;
            var frame = new_node_button.frame;
            var click = find("html/event/click", { element: new_node_tag });
            // @FIXME: dependents of aggregates are busted due to stratification (?).
            // If we try to use ix directly for scanning we get no result.
            // If we try to use it after an expression for scanning we get no filtering at all.
            // Luckily in this case we needed it in a choose, which seems to stratify correctly.
            // @FIXME: Aggregates in chooses don't filter adequately without context.
            // We work around it for now by providing enough context in the choose branch for the aggregate to use.
            var ix = choose(function () {
                return [gather(frame.node).count() + 1];
            }, function () { return 1; })[0];
            return [
                new_node_button.remove("open"),
                frame.add("node", [
                    record("editor/query-node", "editor/root-node", {
                        type: "join",
                        sort: ix,
                        name: client_tag,
                        query_tag: client_tag,
                    })
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Block Query Canvas
        //--------------------------------------------------------------------
        editor
            .block("Draw molecules as hex grids of atoms.", function (_a) {
            var find = _a.find, record = _a.record;
            var canvas_elem = find("editor/block/query-canvas");
            var editor = canvas_elem.editor;
            var active_frame = editor.active_frame;
            active_frame.type == "query";
            var molecule = find("editor/molecule", { editor: editor, frame: active_frame, positioned: "true" });
            var atom = molecule.atom;
            var side = 30;
            var gap = 3;
            return [
                canvas_elem.add({ tag: "shape/hex-grid", side: side, gap: gap }),
                canvas_elem.add("cell", [
                    record("shape/hexagon", "editor/atom/cell", { atom: atom, molecule: molecule, side: side, x: atom.x, y: atom.y, fillStyle: "white", lineWidth: 2, strokeStyle: "#ccc" }).add("content", [
                        record("ui/text", { atom: atom, molecule: molecule, text: atom.node.label + " " + molecule.sort, style: record({ color: atom.node.color }) })
                    ])
                ])
            ];
        })
            .block("Show molecule infobox when open.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var molecule = find("editor/molecule", { open: "true" });
            var atom = molecule.atom, editor = molecule.editor;
            molecule.frame == editor.active_frame;
            var canvas_elem = find("editor/block/query-canvas", { editor: editor });
            var field = atom.field;
            var _b = lookup(field), attribute = _b.attribute, value = _b.value;
            return [
                canvas_elem.add("children", [
                    record("ui/column", "editor/molecule/infobox", { sort: molecule.sort, molecule: molecule }).add("children", [
                        record("ui/text", { text: "Molecule " + molecule.sort }),
                        record("ui/row", { sort: atom.sort, molecule: molecule, atom: atom }).add("children", [
                            record("ui/text", { sort: 0, text: atom.node.name + " {" }),
                            record("ui/text", { sort: atom.sort, text: " " + attribute + ": " + value + " " }),
                            record("ui/text", { sort: Infinity, text: "}" }),
                        ])
                    ])
                ])
            ];
        });
    };
    EditorWatcher.prototype.attachDataEditor = function (editor) {
        editor.block("When the active frame is a data editor, inject the data editor UI.", function (_a) {
            var find = _a.find, union = _a.union, record = _a.record;
            var content = find("editor/block/content");
            var editor = content.editor;
            editor.active_frame.type == "data";
            return [
                content.add("children", [
                    // record("editor/block/data-tree", "ui/column", {editor}),
                    record("editor/block/data-canvas", "ui/column", { editor: editor })
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Data toolbar
        //--------------------------------------------------------------------
        editor
            .block("Draw the data editor toolbar.", function (_a) {
            var find = _a.find, record = _a.record;
            var canvas = find("editor/block/data-canvas");
            var editor = canvas.editor;
            return [
                canvas.add("children", [
                    record("editor/data/toolbar", "ui/column", { sort: 0 }).add("children", [
                        record("editor/data/toolbar/select", "editor/tool", "ui/button", { editor: editor, icon: "arrow-up-c", text: "select", tool: "select", class: "flat" }),
                        record("editor/data/toolbar/add", "editor/tool", "ui/button", { editor: editor, icon: "plus", text: "add", tool: "add", class: "flat" }),
                        record("editor/data/toolbar/remove", "editor/tool", "ui/button", { editor: editor, icon: "minus", text: "erase", tool: "erase", class: "flat" }),
                        record("editor/data/toolbar/save", "editor/tool", "ui/button", { editor: editor, icon: "play", text: "save", tool: "save", class: "flat" }),
                    ])
                ])
            ];
        })
            .commit("Clicking a tool activates it.", function (_a) {
            var find = _a.find, record = _a.record;
            var tool = find("editor/tool");
            find("html/event/click", { element: tool });
            return [
                tool.add("tag", "editor/active")
            ];
        })
            .commit("Clicking with a tool disables it.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            var tool = find("editor/tool", "editor/active", { editor: editor });
            find("html/event/click");
            return [
                tool.remove("tag", "editor/active")
            ];
        })
            .commit("The save tool automatically disables after saving.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root", { tool: "save" });
            var tool = find("editor/data/toolbar/save", "editor/active", { editor: editor });
            return [
                tool.remove("tag", "editor/active")
            ];
        })
            .block("The editor tool is the tool currently marked active.", function (_a) {
            var find = _a.find;
            var _b = find("editor/tool", "editor/active"), tool = _b.tool, editor = _b.editor;
            return [editor.add("tool", tool)];
        });
        //--------------------------------------------------------------------
        // Data canvas
        //--------------------------------------------------------------------
        editor
            .block("Draw molecules as hex grids of atoms.", function (_a) {
            var find = _a.find, record = _a.record;
            var canvas_elem = find("editor/block/data-canvas");
            var editor = canvas_elem.editor;
            var active_block = editor.active_block;
            var frame = active_block.storyboard;
            frame.type == "query";
            var molecule = find("editor/molecule", { editor: editor, frame: frame, positioned: "true" });
            var atom = molecule.atom;
            var side = 30;
            var gap = 5;
            return [
                canvas_elem.add("children", [
                    record("shape/hex-grid", { frame: frame, side: side, gap: gap }).add("cell", [
                        record("shape/hexagon", "editor/atom/cell", { atom: atom, molecule: molecule, side: side, x: atom.x, y: atom.y, fillStyle: "white", lineWidth: 2, strokeStyle: "#ccc" }).add("content", [
                            record("ui/text", { atom: atom, molecule: molecule, text: atom.node.label + " " + molecule.sort, style: record({ color: atom.node.color }) })
                        ])
                    ])
                ])
            ];
        })
            .block("Data molecule infobox.", function (_a) {
            var find = _a.find, record = _a.record;
            var infobox = find("editor/data/molecule/infobox");
            var editor = infobox.editor, molecule = infobox.molecule;
            var atom = molecule.atom;
            var node = atom.node;
            return [
                infobox.add({ tag: "ui/column", sort: molecule.sort, molecule: molecule }).add("children", [
                    record("editor/data/node/infobox", { molecule: molecule, node: node })
                ])
            ];
        })
            .block("Data node infobox.", function (_a) {
            var find = _a.find, record = _a.record;
            var node_infobox = find("editor/data/node/infobox");
            var molecule = node_infobox.molecule, node = node_infobox.node;
            var atom = molecule.atom;
            atom.node == node;
            return [
                node_infobox.add({ tag: "ui/column", sort: node.sort }).add("children", [
                    record("editor/data/node/header", "ui/text", { sort: 0, molecule: molecule, node: node, text: node.name }),
                    record("editor/data/atom/infobox", { molecule: molecule, atom: atom })
                ])
            ];
        })
            .block("A node infobox that's adding always displays an empty field row.", function (_a) {
            var find = _a.find, record = _a.record;
            var node_infobox = find("editor/data/node/infobox", "editor/data/adding");
            var molecule = node_infobox.molecule, node = node_infobox.node;
            return [
                node_infobox.add("children", [
                    record("editor/data/field/row", { sort: "zzzz", molecule: molecule, node: node })
                ])
            ];
        })
            .block("Data atom infobox.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var atom_infobox = find("editor/data/atom/infobox");
            var molecule = atom_infobox.molecule, atom = atom_infobox.atom;
            var node = atom.node, field = atom.field;
            var _b = lookup(field), attribute = _b.attribute, value = _b.value;
            return [
                atom_infobox.add({ tag: "ui/column", sort: atom.sort, node: node }).add("children", [
                    record("editor/data/field/row", { molecule: molecule, node: node, atom: atom, attribute: attribute }).add({ field: field, value: value })
                ])
            ];
        })
            .block("Data field row.", function (_a) {
            var find = _a.find, record = _a.record;
            var field_row = find("editor/data/field/row");
            var molecule = field_row.molecule, node = field_row.node, attribute = field_row.attribute, value = field_row.value;
            return [
                field_row.add({ tag: "ui/row", sort: attribute }).add("children", [
                    record("editor/data/field/attribute", "ui/text", { sort: 1, field_row: field_row, attribute: attribute, text: attribute }),
                    record("editor/data/field/value-set", "ui/column", { sort: 2, field_row: field_row, attribute: attribute }).add("children", [
                        record("editor/data/field/value", "ui/text", { sort: value, field_row: field_row, attribute: attribute, value: value, text: value })
                    ])
                ])
            ];
        })
            .block("A data field row without an attribute is a new field row.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var field_row = find("editor/data/field/row");
            not(function () { return field_row.attribute; });
            var molecule = field_row.molecule, node = field_row.node;
            return [
                field_row.add({ tag: ["editor/data/field/new-row", "ui/row"] }).add("children", [
                    record("editor/data/field/attribute", "editor/data/field/new-attribute", "ui/input", { sort: 1, field_row: field_row, placeholder: "attribute..." }),
                    record("editor/data/field/value-set", "ui/column", { sort: 2, field_row: field_row }).add("children", [
                        record("editor/data/field/value", "editor/data/field/new-value", "ui/input", { field_row: field_row, text: "value..." })
                    ])
                ])
            ];
        })
            .block("A field row that's adding always displays an empty value at the bottom of it's value set.", function (_a) {
            var find = _a.find, record = _a.record;
            var field_row = find("editor/data/field/row", "editor/data/adding");
            var value_set = field_row.children;
            value_set.tag == "editor/data/field/value-set";
            return [
                value_set.add("children", [
                    record("editor/data/field/value", "editor/data/field/new-value", "ui/input", { sort: "zzzz", field_row: field_row, placeholder: "value..." })
                ])
            ];
        })
            .block("new values dump into their field row's adding_value attribute.", function (_a) {
            var find = _a.find;
            var new_value = find("editor/data/field/new-value");
            var field_row = new_value.field_row, value = new_value.value;
            return [field_row.add("adding_value", value)];
        })
            .block("new attributes dump into their field row's adding_attribute attribute.", function (_a) {
            var find = _a.find;
            var new_attribute = find("editor/data/field/new-attribute");
            var field_row = new_attribute.field_row, value = new_attribute.value;
            return [field_row.add("adding_attribute", value)];
        })
            .block("Show molecule infobox when open.", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var molecule = find("editor/molecule", { open: "true" });
            var editor = molecule.editor;
            var canvas_elem = find("editor/block/data-canvas", { editor: editor });
            return [canvas_elem.add("children", record("editor/data/molecule/infobox", { editor: editor, molecule: molecule }))];
        });
        //--------------------------------------------------------------------
        // Data editing
        //--------------------------------------------------------------------
        editor
            .block("Clicking an erased target with the add tool undoes the erase.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var removed = find("editor/data/erasing");
            find("html/event/click", { element: removed });
            var editor = choose(function () { return removed.editor; }, function () { return removed.molecule.editor; }, function () { return removed.field_row.molecule.editor; })[0];
            editor.tool == "add";
            return [
                record("editor/cancel-action", { editor: editor, element: removed, cancelled: "editor/data/erasing" })
            ];
        })
            .block("Clicking an added target with the erase tool undoes the add.", function (_a) {
            var find = _a.find, choose = _a.choose, record = _a.record;
            var added = find("editor/data/adding");
            find("html/event/click", { element: added });
            var editor = choose(function () { return added.editor; }, function () { return added.molecule.editor; }, function () { return added.field_row.molecule.editor; })[0];
            editor.tool == "erase";
            return [
                record("editor/cancel-action", { editor: editor, element: added, cancelled: "editor/data/adding" })
            ];
        })
            .commit("If an add and erase cancelled, remove the cancelled tag on whatever element it happened on.", function (_a) {
            var find = _a.find;
            var cancel = find("editor/cancel-action");
            var element = cancel.element, cancelled = cancel.cancelled;
            return [element.remove("tag", cancelled)];
        })
            .commit("Clicking an infobox attribute with the erase tool marks its field for deletion.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var target = find("editor/data/field/attribute");
            find("html/event/click", { element: target });
            var field_row = target.field_row;
            var molecule = field_row.molecule;
            molecule.editor.tool == "erase";
            not(function () { return find("editor/cancel-action", { editor: molecule.editor }); });
            return [
                field_row.add("tag", "editor/data/erasing")
            ];
        })
            .commit("Clicking an infobox value with the erase tool marks it for deletion.", function (_a) {
            var find = _a.find, not = _a.not, record = _a.record;
            var target = find("editor/data/field/value");
            find("html/event/click", { element: target });
            var field_row = target.field_row;
            var molecule = field_row.molecule;
            molecule.editor.tool == "erase";
            not(function () { return find("editor/cancel-action", { editor: molecule.editor }); });
            return [
                target.add("tag", "editor/data/erasing")
            ];
        })
            .commit("Clicking an infobox value with the add tool marks the field row as adding.", function (_a) {
            var find = _a.find, not = _a.not, choose = _a.choose, record = _a.record;
            var target = find("editor/data/field/value-set");
            var field_row = find("editor/data/field/row", { children: target });
            find("html/event/click", { element: target });
            var molecule = field_row.molecule;
            molecule.editor.tool == "add";
            not(function () { return find("editor/cancel-action", { editor: molecule.editor }); });
            return [
                field_row.add("tag", "editor/data/adding")
            ];
        })
            .commit("Clicking an infobox attribute with the add tool marks its node as adding.", function (_a) {
            var find = _a.find, not = _a.not, choose = _a.choose, record = _a.record;
            var target = find("editor/data/field/attribute");
            find("html/event/click", { element: target });
            var _b = target.field_row, molecule = _b.molecule, node = _b.node;
            var node_infobox = find("editor/data/node/infobox", { node: node });
            molecule.editor.tool == "add";
            return [
                node_infobox.add("tag", "editor/data/adding")
            ];
        });
        //--------------------------------------------------------------------
        // Data save changes
        //--------------------------------------------------------------------
        editor
            .commit("When the editor's current tool is save, commit all pending value erases.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root", { tool: "save" });
            var erasing = find("editor/data/erasing", "editor/data/field/value");
            var active_block = editor.active_block;
            var molecule = erasing.molecule, node = erasing.node, attribute = erasing.attribute, value = erasing.value;
            molecule.editor == editor;
            return [
                active_block.add("data_output", record("editor/data/erase/value", { node: node, attribute: attribute, value: value })),
                erasing.remove("tag", "editor/data/erasing")
            ];
        })
            .commit("When the editor's current tool is save, commit all pending attribute erases.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root", { tool: "save" });
            var erasing = find("editor/data/erasing", "editor/data/field/attribute");
            var active_block = editor.active_block;
            var molecule = erasing.molecule, node = erasing.node, attribute = erasing.attribute;
            molecule.editor == editor;
            return [
                active_block.add("data_output", record("editor/data/erase/attribute", { node: node, attribute: attribute })),
                erasing.remove("tag", "editor/data/erasing")
            ];
        })
            .commit("When the editor's current tool is save, commit all pending value adds.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root", { tool: "save" });
            var active_block = editor.active_block;
            var adding = find("editor/data/adding", "editor/data/field/row");
            var molecule = adding.molecule, node = adding.node, attribute = adding.attribute, adding_value = adding.adding_value;
            molecule.editor == editor;
            return [
                active_block.add("data_output", record("editor/data/add/value", { node: node, attribute: attribute, value: adding_value })),
                adding.remove("tag", "editor/data/adding")
            ];
        })
            .commit("When the editor's current tool is save, commit all pending attribute adds.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root", { tool: "save" });
            var active_block = editor.active_block;
            var adding = find("editor/data/adding", "editor/data/node/infobox");
            var molecule = adding.molecule, node = adding.node;
            molecule.editor == editor;
            var new_row = find("editor/data/field/row", { node: node });
            var adding_attribute = new_row.adding_attribute, adding_value = new_row.adding_value;
            return [
                active_block.add("data_output", record("editor/data/add/value", { node: node, attribute: adding_attribute, value: adding_value })),
                adding.remove("tag", "editor/data/adding")
            ];
        })
            .block("What are the data outputs?", function (_a) {
            var find = _a.find, lookup = _a.lookup, record = _a.record;
            var editor = find("editor/root");
            var active_block = editor.active_block;
            var data_output = active_block.data_output;
            var _b = lookup(data_output), attribute = _b.attribute, value = _b.value;
            return [
                record("ui/column", "debuggeroo", { active_block: active_block }).add("children", [
                    record("ui/text", { sort: 0, text: active_block.name }),
                    record("ui/row", { sort: data_output }).add("children", [
                        record("ui/text", { sort: attribute, data_output: data_output, text: " " + attribute + ": " + value + " " })
                    ])
                ])
            ];
        });
        //--------------------------------------------------------------------
        // Data block generation
        //--------------------------------------------------------------------
        editor
            .block("When the editor's current tool is save, create a block representing the changes.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            var active_block = editor.active_block;
            return [
                active_block.add("data_block", [
                    record("editor/data/block", "eve/compiler/block", { editor: editor, block: active_block, name: "Data block", type: "block" })
                ])
            ];
        })
            .commit("Clear the old block contents.", function (_a) {
            var find = _a.find;
            var editor = find("editor/root", { tool: "save" });
            var block = editor.block;
            var data_block = block.data_block;
            var variable = find("editor/data/node/entity");
            return [
                data_block.constraint.remove(),
                data_block.constraint.attribute.remove(),
                variable.remove()
            ];
        })
            .commit("Mark the datablock to be rebuilt on save.", function (_a) {
            var find = _a.find;
            var editor = find("editor/root", { tool: "save" });
            var block = editor.block;
            var data_block = block.data_block;
            return [data_block.add("rebuild", "true")];
        })
            .commit("Clear rebuild off the datablock.", function (_a) {
            var find = _a.find;
            var editor = find("editor/root");
            var block = editor.block;
            var data_block = block.data_block;
            data_block.rebuild == "true";
            return [data_block.remove("rebuild", "true")];
        })
            .commit("Copy all the records from the molecule watch.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            var block = editor.block;
            var data_block = block.data_block;
            data_block.rebuild;
            var atom_record = find("editor/atom/record");
            var node = atom_record.node, attr = atom_record.attribute;
            var entity_var;
            return [
                entity_var = record("editor/data/node/entity", "eve/compiler/var", { node: node }),
                data_block.add("constraint", record("eve/compiler/record", { node: node, record: entity_var }).add("attribute", [
                    record({ attribute: attr.attribute, value: attr.value })
                ])),
            ];
        })
            .block("Add value outputs.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            var block = editor.block;
            var data_block = block.data_block, data_output = block.data_output;
            // data_block.rebuild;
            data_output.tag == "editor/data/add/value";
            var node = data_output.node, attribute = data_output.attribute, value = data_output.value;
            var node_var = find("editor/data/node/entity", { node: node });
            return [
                data_block.add("constraint", [
                    record("eve/compiler/output", { node: node, record: node_var }).add("attribute", [
                        record({ attribute: attribute, value: value })
                    ])
                ])
            ];
        })
            .commit("Erase value outputs.", function (_a) {
            var find = _a.find, record = _a.record;
            var editor = find("editor/root");
            var block = editor.block;
            var data_block = block.data_block, data_output = block.data_output;
            data_block.rebuild;
            data_output.tag == "editor/data/erase/value";
            var node = data_output.node, attribute = data_output.attribute, value = data_output.value;
            var node_var = find("editor/data/node/entity", { node: node });
            return [
                data_block.add("constraint", [
                    record("eve/compiler/remove", "eve/compiler/output", { node: node, record: node_var }).add("attribute", [
                        record({ attribute: attribute, value: value })
                    ])
                ])
            ];
        });
    };
    EditorWatcher.prototype.initEditor = function () {
        //--------------------------------------------------------------------
        // Kick it off
        //--------------------------------------------------------------------
        var input = [
            [EDITOR_ID, "tag", "editor/root"],
            [STYLE_ID, "tag", "html/element"],
            [STYLE_ID, "tagname", "link"],
            [STYLE_ID, "rel", "stylesheet"],
            [STYLE_ID, "href", "assets/css/editor.css"],
        ];
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 1, ix: 1, x: 0, y: 0 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 1, ix: 2, x: 1, y: 0 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 1, ix: 3, x: 1, y: 1 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 1, ix: 4, x: 0, y: 1 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 1, ix: 5, x: -1, y: 0 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 1, ix: 6, x: 0, y: -1 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 1, ix: 7, x: 1, y: -1 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 0, ix: 1, x: 0, y: 0 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 0, ix: 2, x: 1, y: 0 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 0, ix: 3, x: 0, y: 1 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 0, ix: 4, x: -1, y: 1 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 0, ix: 5, x: -1, y: 0 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 0, ix: 6, x: -1, y: -1 });
        watcher_1.appendAsEAVs(input, { tag: "spiral", row: 0, ix: 7, x: 0, y: -1 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 0 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 1 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 2 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 3 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 4 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 5 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 6 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 7 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 8 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 9 });
        watcher_1.appendAsEAVs(input, { tag: "range", ix: 10 });
        // appendAsEAVs(input, {tag: "node-color", ix: 1, color: "red"});
        // appendAsEAVs(input, {tag: "node-color", ix: 2, color: "orange"});
        // appendAsEAVs(input, {tag: "node-color", ix: 3, color: "yellow"});
        // appendAsEAVs(input, {tag: "node-color", ix: 4, color: "green"});
        // appendAsEAVs(input, {tag: "node-color", ix: 5, color: "blue"});
        // appendAsEAVs(input, {tag: "node-color", ix: 6, color: "indigo"});
        // appendAsEAVs(input, {tag: "node-color", ix: 7, color: "violet"});
        // appendAsEAVs(input, {tag: "node-color", ix: 8, color: "light gray"});
        // appendAsEAVs(input, {tag: "node-color", ix: 9, color: "dark gray"});
        watcher_1.appendAsEAVs(input, { tag: "node-color", ix: 1, color: "#9926ea" });
        watcher_1.appendAsEAVs(input, { tag: "node-color", ix: 2, color: "#6c86ff" });
        watcher_1.appendAsEAVs(input, { tag: "node-color", ix: 3, color: "red" });
        watcher_1.appendAsEAVs(input, { tag: "node-color", ix: 4, color: "orange" });
        watcher_1.appendAsEAVs(input, { tag: "node-color", ix: 5, color: "green" });
        watcher_1.appendAsEAVs(input, { tag: "node-color", ix: 6, color: "indigo" });
        this.editor.inputEavs(input);
        console.log(this);
    };
    EditorWatcher.prototype.fixtureEditor = function () {
        /*
         * [#editor/root
         *  active_block // Block being edited
         *  active_frame // Current frame of the active_block
         *  block        // All blocks in program
         * ]
         *
         * [#editor/block
         *  name
         *  description
         *  nav_tag          // Navigation tags that relate to this block
         *  storyboard       // All frames of the block
         *  molecule_watch C // Watch block(s) producing molecules for the given block
         *  data_output      // All operations being applied to the current data
         *  data_block     C // Compiled blocks that run the data_output operations
         * ]
         *
         * [#editor/frame
         *  type // Editor type associated with this frame ("query", "data", etc.)
         *  sort // Order of this frame in the parent block's storyboard
         * ]
         * [#editor/frame type: "query"
         *  node // All nodes contributing to this query
         * ]
         *
         * [#editor/query-node
         *  type               // Join, expression, etc.
         *  sort               // Order of this node in the parent frame's node list
         *  name               // Human readable node name
         *  query_tag        ? // Matches records IFF they have the given tag
         *  query_field      ? // Matches records IFF they have the given attributes
         *  parent_node      ?
         *  parent_attribute ? // Matches records IFF they are in the value set of parent_attribute on a record matching parent_node
         *  label            C // Short label for the node
         *  color            C
         * ]
         *
         * [#editor/molecule // Macro compiled
         *  editor
         *  frame
         *  node             // The root node that defines the identity of the molecule
         *  root_atom_record // The entity ID of the record defining the identity of th molecule
         *  atom             // The set of atoms composing the molecule
         * ]
         *
         * [#editor/atom // Macro compiled
         *  node
         *  molecule
         *  record
         *  field: [ // AVs matching the query_field attributes for this atom's node
         *   <query_field>: value
         *  ]
         * ]
         *
         */
        var fixture = [
            [EDITOR_ID, "block", BLOCK_PPL_W_BOATS_ID],
            [EDITOR_ID, "block", BLOCK_BOAT_TYPES_ID],
            [EDITOR_ID, "active_block", BLOCK_PPL_W_BOATS_ID],
            [EDITOR_ID, "active_frame", FRAME_PPL_W_BOATS_QUERY_ID]
        ];
        watcher_1.appendAsEAVs(fixture, { name: "Marina" }, TAG_MARINA_ID);
        watcher_1.appendAsEAVs(fixture, { name: "Marinara" }, TAG_MARINARA_ID);
        watcher_1.appendAsEAVs(fixture, {
            tag: "editor/block",
            nav_tag: [TAG_MARINA_ID, TAG_MARINARA_ID],
            name: "People with boats",
            description: "Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's standard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged.",
            storyboard: [
                watcher_1.appendAsEAVs([], {
                    tag: "editor/frame",
                    type: "query",
                    sort: 1,
                    node: []
                }, FRAME_PPL_W_BOATS_QUERY_ID),
                watcher_1.appendAsEAVs([], {
                    tag: "editor/frame",
                    type: "data",
                    sort: 2,
                })
            ],
        }, BLOCK_PPL_W_BOATS_ID);
        watcher_1.appendAsEAVs(fixture, {
            tag: "editor/block",
            nav_tag: TAG_MARINA_ID,
            name: "Boat types",
            description: "It is a long established fact that a reader will be distracted by the readable content of a page when looking at its layout. The point of using Lorem Ipsum is that it has a more-or-less normal distribution of labels, as opposed to using 'Content here, content here', making it look like readable English."
        }, BLOCK_BOAT_TYPES_ID);
        this.editor.inputEavs(fixture);
    };
    return EditorWatcher;
}(watcher_1.Watcher));
watcher_1.Watcher.register("editor", EditorWatcher);
//# sourceMappingURL=editor.js.map